import asyncio
import sys
from src.client import Client

async def run():
    client = Client()
    await client.start()
    if client.do_action=='1':
        await client.query('/exit')
        
    print('已退出程序')

def main():
    if sys.version_info.minor>=10:
        asyncio.run(run())
    else:
        loop = asyncio.get_event_loop()
        loop.run_until_complete(run())
    
    




import asyncio
import subprocess
from concurrent.futures.thread import ThreadPoolExecutor
import functools
from getpass import getpass
import logging
import signal
import hashlib
import random
import string

log = logging.getLogger(__name__)

signals = {
    k: v for v, k in signal.__dict__.items()
    if v.startswith("SIG") and not v.startswith("SIG_")
}

def already_runing(_file_):
    proc = subprocess.Popen(["pgrep", "-f", _file_], stdout=subprocess.PIPE)
    std = proc.communicate()
    if len(std[0].decode()) > 1:
        exit("Already runing")


async def ainput(prompt: str = "", *, hide: bool = False):
    """Just like the built-in input, but async"""
    with ThreadPoolExecutor(1) as executor:
        func = functools.partial(getpass if hide else input, prompt)
        return await asyncio.get_event_loop().run_in_executor(executor, func)


def md5(value: str, salt: str = '') -> str:
    return str(hashlib.md5((value+salt).encode(encoding='UTF-8')).hexdigest()).lower()


def rdValue(len: int = 20, is_digits: bool = None, is_letter: bool = None) -> str:
    '''生成随机字符

    is_digits = True 时只生成数字

    is_letter = True 时只生成字母

    否则生成字母和数字组合
    '''
    if is_digits:
        chats = string.digits
    elif is_letter:
        chats = string.ascii_letters
    else:
        chats = string.ascii_letters + string.digits
    return ''.join(random.sample(chats, len))


stop = False
# 自定义信号处理函数

def my_handler(signum, frame):
    global stop
    stop = True
    
    print('\n等待上一任务处理后即结束....')
    


# 设置相应信号处理的handler
signal.signal(signal.SIGINT, my_handler)
signal.signal(signal.SIGHUP, my_handler)
signal.signal(signal.SIGTERM, my_handler)


async def idle(cb=None,exit:bool=None):
    if exit:
        global stop
        stop = True
        return
    while True:
        try:
            if stop:
                # 中断时需要处理的代码
                break
            elif cb:
                await cb()
        except Exception as e:
            print(str(e))
            break



import asyncio
import configparser
import functools
import json
import logging
import os
import platform
from concurrent.futures.thread import ThreadPoolExecutor
from contextlib import closing
from getpass import getpass
from typing import Any, Callable, List
import sys
import requests
from fake_useragent import UserAgent
from qcloud_cos import CosConfig, CosS3Auth, CosS3Client
from qcloud_cos.cos_exception import CosClientError, CosServiceError
from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor

from .object import Object
from .utils import md5, rdValue

log = logging.getLogger(__name__)
#logging.basicConfig(level=logging.INFO, stream=sys.stdout)

class Result(Object):
    def __init__(self, success: bool, msg: str, data: Any=None, **kwargs) -> None:
        self.success = success
        self.msg = msg or ''
        self.data = data
        self.code = kwargs.get('code') or 0


async def ainput(prompt: str = "", *, hide: bool = False):
    """Just like the built-in input, but async"""
    with ThreadPoolExecutor(1) as executor:
        func = functools.partial(getpass if hide else input, prompt)
        return await asyncio.get_event_loop().run_in_executor(executor, func)

host_uri = 'https://www.shortvideo.work/api/xiaozhu' #'http://192.168.5.84:8012/api/xiaozhu'

#https://gist.github.com/kazqvaizer/4cebebe5db654a414132809f9f88067b
def multipartify(data, parent_key=None, formatter: callable = None) -> dict:
    if formatter is None:
        def formatter(v): return (None, v)  # Multipart representation of value

    if type(data) is not dict:
        return {parent_key: formatter(data)}

    converted = []

    for key, value in data.items():
        current_key = key if parent_key is None else f"{parent_key}[{key}]"
        if type(value) is dict:
            converted.extend(multipartify(
                value, current_key, formatter).items())
        elif type(value) is list:
            for ind, list_value in enumerate(value):
                iter_key = f"{current_key}[{ind}]"
                converted.extend(multipartify(
                    list_value, iter_key, formatter).items())
        else:
            converted.append((current_key, formatter(value)))

    return dict(converted)

config_path = os.path.join(os.path.dirname(__file__),'config.ini')

mode = {'api':'API发文','imitate':'模拟发文','webupload':'网页上传视频发文','webpublish':'网页直接发文'}

class Method:
    async def query(self, url: str, **kwargs):
        def value_string(data: dict):
            array = []
            for k in sorted(data):
                v = data[k]
                if isinstance(v, bool):
                    array.append(f'{k}={str(v).lower()}')
                elif isinstance(v, dict):
                    array.append(f'{k}={value_string(v)}')
                elif(isinstance(v, list)):
                    for i in v:
                        array.append(f'{k}=[{value_string(i)}]')
                else:
                    array.append(f'{k}={v}')
            return '&'.join(array)

        try:
            nonce = rdValue(10)
            data = value_string(kwargs)
            sign = md5(f'{url}{data}{nonce}', self.appsecret)
            
            url = f'{self.host}{url}?appid={self.appid}&nonce={nonce}&sign={sign}&version={self.version}'
            
            headers = {'Content-Type': 'application/json'}
            
            response = requests.post(url, data=json.dumps(
                kwargs), headers=headers) if kwargs else requests.get(url, headers=headers)

            if(response.status_code == 200):
                value = response.json()
                res = Result(value.get('success'),
                             value.get('msg'), value.get('data'))
                if(value.get('code') == 401):
                    res.code = 401
                return res
            else:
                return Result(False, str(response.status_code), None, code=response.status_code)

        except Exception as e:
            return Result(False, str(e), None)

    async def settings(self):
        
        config = configparser.ConfigParser()
        if not os.path.exists(config_path):
            config.add_section("settings")
            config.set("settings", "desc", "请不在擅自修改配置文件，以免引发异常！")
            config.set("settings", "version", self.version)
            config.set("settings", "host", host_uri)
            config.set("settings","mode","api") #'webupload' | 'webpublish' | 'api' | 'imitate'

            config.write(open(config_path, "w"))

        config.read(config_path)

        def get(name: str, default: Any = ''):
            return config.get('settings', name, fallback=default)

        def set(name: str, value: Any):
            config.set('settings', name, value)

        if(get('version') != self.version):
            set('version', self.version)
            config.write(open(config_path, "w"))

        if not get('host'):
            set('host', host_uri)
            config.write(open(config_path, "w"))

        self.host = get('host')
        self.mode = get('mode')

        while(not self.appid or not self.appsecret or not self.workdir):
            is_save = False
            self.appid = get('appid')
            if not self.appid:
                appid = await ainput("请输入AppID：")
                if appid == 'exit':
                    break
                elif not appid:
                    continue
                else:
                    self.appid = appid
                    set('appid', self.appid)
                    is_save = True

            self.appsecret = get('appsecret')
            if not self.appsecret:
                appsecret = await ainput("请输入AppSecret：")
                if appsecret == 'exit':
                    break
                elif not appsecret:
                    continue
                else:
                    self.appsecret = appsecret
                    set('appsecret', self.appsecret)
                    is_save = True

            self.workdir = get('workdir')
            if not self.workdir:
                workdir = await ainput("请输入工作目录：")
                if workdir == 'exit':
                    break
                elif not workdir:
                    continue
                else:
                    if os.path.exists(workdir):
                        self.workdir = workdir
                        set('workdir', self.workdir)
                        is_save = True
                    else:
                        print(f'不存在的目录：{workdir}')
                        continue

            if is_save:
                res = await self.query('/checkSecret', secret=md5(self.appsecret))
                if res.success:
                    config.write(open(config_path, "w"))
                else:
                    print(res.msg)

            break

        return bool(self.appid and self.appsecret and self.workdir)

    async def home(self, callback: Callable[[str], None]):
        def render():
            sysstr = platform.system()
            os.system('cls' if sysstr == 'Windows' else 'clear')
            options = [
                f'\033[31;42m{" "*10}短视频工作平台之客户端({self.version}){" "*10}\033[0m',
                f'{" "*10}www.shortvideo.work\n',f'配置路径：{config_path}\n发文模式：{mode.get(self.mode) or self.mode}\n',
                '1 开始执行任务\t2 只执行一次任务',
                '3 修改AppSecret',
                f'4 修改服务器地址（{self.host}）',
                f'5 修改工作目录（{self.workdir}）',
                '0 退出程序',
                '\n请选择相应操作的序号：'
            ]
            return '\n'.join(options)

        async def setWorkdir():
            workdir = None
            while not workdir:
                workdir = await ainput('请输入新的工作目录（输入0则返回）：')
                if workdir == '0' or workdir == self.workdir:
                    await main()
                elif workdir:
                    if os.path.exists(workdir):
                        config = configparser.ConfigParser()
                        config.read(config_path)
                        config.set('settings', 'workdir', workdir)
                        config.write(open(config_path, "w"))
                        self.workdir = workdir
                        await main()
                    else:
                        print(f'{workdir} 无效')
                        workdir = None
                else:
                    continue

        async def setSecret():
            appsecret = None
            while not appsecret:
                appsecret = await ainput('请输入新的AppSecret（0则返回）：')
                if appsecret == '0':
                    await main()
                elif not appsecret:
                    continue
                else:
                    res = await self.query('/checkSecret', secret=md5(appsecret))
                    if res.success:
                        config = configparser.ConfigParser()
                        config.read(config_path)
                        config.set('settings', 'appsecret', appsecret)
                        config.write(open(config_path, "w"))
                        self.appsecret = appsecret
                        await main()
                    else:
                        print(res.msg)
                        appsecret = None

        async def setHost():
            host = None
            while not host:
                host = await ainput('请输入服务器地址（0则返回）：')
                if host == '0' or host == self.host:
                    await main()
                elif not host:
                    continue
                else:
                    oldHost = self.host
                    self.host = host
                    res = await self.query('/checkSecret', secret=md5(self.appsecret))
                    if res.success:
                        config = configparser.ConfigParser()
                        config.read(config_path)
                        config.set('settings', 'host', self.host)
                        config.write(open(config_path, "w"))
                        await main()
                    else:
                        self.host = oldHost
                        print(res.msg)
                        host = None

        async def main():
            value = None
            while value not in ['0','1', '2', '3', '4', '5']:
                value = await ainput(render())

            if value == '1' or value == '2':
                try:
                    await callback(value)
                except Exception as e:
                    log.error(e,exc_info=True)

            elif value == '3':
                await setSecret()
            elif value == '4':
                await setHost()
            elif value == '5':
                await setWorkdir()
            elif value == '0':
                pass

        await main()

    async def download(self, savepath: str, src: str, callback: Callable[[int, int, int], None]):
        try:
            with closing(requests.get(src, stream=True)) as response:
                chunk_size = 1024  # 单次请求最大值
                total = int(response.headers['content-length'])  # 内容体总大小
                size = 0
                with open(savepath, "wb") as file:
                    for data in response.iter_content(chunk_size=chunk_size):
                        file.write(data)
                        size = size + len(data)
                        progress = (size / total) * 100
                        callback(progress, size, total)
        except:
            return False
        else:
            return True

    async def to_cos(self, cos: dict, filepath: str, accept: str, callback):
        def cb(monitor):
            progress = (monitor.bytes_read / monitor.len) * 100
            callback(progress, monitor.bytes_read, monitor.len)

        bucket = cos.get('bucket')
        key = cos.get('filekey')
        region = cos.get('region')
        StartTime = cos.get('startTime')
        expiredTime = cos.get('expiredTime')
        credentials = cos.get('credentials') or []
        secret_id = credentials.get('tmpSecretId')
        secret_key = credentials.get('tmpSecretKey')
        token = credentials.get('sessionToken')
        url = f'https://{bucket}.cos.ap-shanghai.myqcloud.com{key}'

        config = CosConfig(Region=region, SecretId=secret_id,
                           SecretKey=secret_key, Token=token)
        client = CosS3Client(config)

        def cb(monitor):
            progress = (monitor.bytes_read / monitor.len) * 100
            callback(progress, monitor.bytes_read, monitor.len)

        fields = {
            'key': key,
            'Content-Type': accept,
            'file': (key, open(filepath, 'rb'), accept)
        }
        e = MultipartEncoder(fields)
        m = MultipartEncoderMonitor(e, cb)
        headers = {
            'Host': f'{bucket}.cos.ap-shanghai.myqcloud.com',
            'Content-Type': m.content_type,
            'x-cos-security-token': token
        }
        headers['Authorization'] = client.get_auth(
            "POST", bucket, key, Expired=expiredTime-StartTime)

        try:
            r = requests.post(url, data=m, headers=headers)

            if(r.status_code == 200 or r.status_code == 204):
                return Result(
                    True, "",
                    {'ETag': r.headers.get('ETag').replace('"', ''), 'Location': r.headers.get(
                        'Location'), 'x-cos-request-id': r.headers.get('x-cos-request-id')}
                )
            else:
                return Result(False, r.text, None)
        except Exception as ex:
            log.error(ex, exc_info=True)
            return Result(False, str(ex), ex)

    async def login(self, mobile: str, password):
        user_agent = UserAgent().random

        url = "https://mp.xiaozhuyouban.com/signin"

        r = requests.get("https://mp.xiaozhuyouban.com/")

        xsrfToken = r.cookies.get("XSRF-TOKEN")

        headers = {
            'authority': 'mp.xiaozhuyouban.com',
            'accept': '*/*',
            'accept-language': 'zh-CN,zh;q=0.9,en;q=0.8,zh-TW;q=0.7',
            'content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'origin': 'https://mp.xiaozhuyouban.com',
            'referer': 'https://mp.xiaozhuyouban.com/',
            'user-agent': user_agent,
            'x-requested-with': 'XMLHttpRequest'
        }
        r = requests.post(
            url,
            data={"xsrfToken": xsrfToken,
                  "mobile": mobile, "password": password,"agree":"on"},
            cookies=r.cookies,
            headers=headers
        )
        if r.status_code == 200:
            data = r.json() or {'code':-1}
            code = data.get('code')
            message = data.get('message') or '登录未知错误'
            if code == 0:
                r.cookies.set('__root_domain_v','.xiaozhuyouban.com')
                return Result(True, '', (user_agent, r.cookies))
            else:
                action = 0
                if '爬虫' in message:
                    action = 401 #全部停止
                return Result(False, message, None,code=action)
        else:
            return Result(False, f'登录失败：{r.status_code}', None)

    def __formatCosData(self, data: dict, is_video: bool):
        result = {'statusCode': 200}
        for key, value in data.items():
            if key == 'Location':
                Location: str = value.replace(
                    'http://', '').replace('https://', '')
                result['Location'] = Location
                if is_video:
                    arr = Location.split('.')
                    result['Bucket'] = arr[0]
                    result['Key'] = '/'.join(Location.split('/')[1:])
            elif key == 'ETag':
                result['ETag'] = f'"{value}"'
            elif key == 'x-cos-request-id':
                result['RequestId'] = value
            elif key == 'UploadId':
                result['UploadId'] = value

        if is_video:
            result['headers'] = {
                'content-type': 'application/xml', 'x-cos-request-id': result['RequestId']}
        else:
            result['headers'] = {
                'content-length': '0', 'etag': result['ETag'], 'x-cos-request-id': result['RequestId']}
        return result

    async def submit(self,
                     mobile: str, password: str,
                     filepath: str,
                     title: str, category: str, source: int, tag: List[str],
                     uploadres_cos: dict,
                     coverres_cos: dict,
                     coverres2_cos: dict = None,
                     coverres1_cos: dict = None,
                     inspiration_cateid: str = None,
                     activity: str = None,
                     desc: str = None
                     ):

        res = await self.login(mobile, password)
        if not res.success:
            return Result(False, res.msg,dict(code=-1,message=res.msg))

        user_agent, cookies = res.data

        xsrfToken = cookies.get('XSRF-TOKEN')

        headers = {
            'authority': 'mp.xiaozhuyouban.com',
            'accept': '*/*',
            'accept-language': 'zh-CN,zh;q=0.9,en;q=0.8,zh-TW;q=0.7',
            'origin': 'https://mp.xiaozhuyouban.com',
            'referer': 'https://mp.xiaozhuyouban.com/content/publish',
            'user-agent': user_agent,
            'x-requested-with': 'XMLHttpRequest',
            'x-csrf-token': xsrfToken,
        }

        payload = {
            'title': title,
            'cover': 0,
            'category': category,
            'source': source,
            'tag': tag,
            'inspiration_cateid': inspiration_cateid or '',
            'activity': activity or '',
            'desc': desc or '',
            'video_cos': self.__formatCosData(uploadres_cos, True),
            'coverinfo_cos': self.__formatCosData(coverres_cos, False)
        }
        if coverres2_cos:
            payload['coverinfo2_cos'] = self.__formatCosData(
                coverres2_cos, False)
        if coverres1_cos:
            payload['coverinfo1_cos'] = self.__formatCosData(
                coverres1_cos, False)

        value = multipartify(payload)
        value["coverfile"] = (f'{title}.jpg', open(filepath, 'rb'), 'image/jpeg')
        
        r = requests.post(
            'https://mp.xiaozhuyouban.com/content/publish',
            files=value, headers=headers, cookies=cookies
        )
        
        if r.status_code == 200:
            data = r.json() or {'code':-1}
            code = data.get('code')
            message = data.get('message') or '发布未知错误'
            return Result(code == 0, message,dict(code=code, message=message))
            if code == 0:
                return Result(True, message,dict(code=code, message=message))
            else:
                action = 0
                if '爬虫' in message or message == '周六，周日限制发文':
                    action = 401 #全部停止
                return Result(False, message,code=action)
        else:
            message = f'请求出错：{r.status_code}'
            return Result(False, message,dict(code=-1,message=message))




from setuptools import setup

packages = ["src"]

requires = [
    "requests==2.28.1",
    "requests_toolbelt==0.9.1",
    "cos_python_sdk_v5==1.9.21",
    "fake_useragent==1.1.1",
    "qcloud_cos==3.3.6"
]


setup(
    name='shortvideo',  # 包名称
    version='1.0.3',  # 包版本
    description='',  # 包详细描述
    long_description='', 
    author='netfere',  # 作者名称
    author_email='netfere@gmail.com',  # 作者邮箱
    url='',   # 项目官网
    packages=packages,    # 项目需要的包
    python_requires=">=3.7",  # Python版本依赖
    install_requires=requires,  # 第三方库依赖
    entry_points={
        'console_scripts':[
            'shortvideo = src.main:main'
        ]
    },
)


import asyncio
import datetime
import os
import re
import math
from typing import Any, Dict, List
import json
import random
from fake_useragent import UserAgent

from .method import Method, Result
from .utils import idle
import requests

version = '1.0.3'


class File:
    def __init__(self, id: int, label: str, md5: str, src: str, size: int, accept: str, name: str, cos: dict = None, **kwargs) -> None:
        self.id = id
        self.label = label
        self.md5 = md5
        self.src = src
        self.size = size
        self.accept = accept
        self.name = name
        self.cos = cos

    @property
    def sizeStr(self):
        value = 0
        unit = ''
        if self.size < 1024:
            value = self.size
            unit = 'B'
        elif self.size < 1024*1024:
            value = self.size/1024
            unit = 'KB'
        elif self.size < 1024*1024*1024:
            value = self.size/1024/1024
            unit = 'MB'
        else:
            value = self.size/1024/1024/1024
            unit = 'GB'
        return f'{math.ceil(value*10)/10}{unit}'


class Client(Method):
    appid: str
    appsecret: str
    host: str
    workdir: str
    index: int
    do_action: str
    mode: str

    def __init__(self):
        self.appid = ''
        self.appsecret = ''
        self.host = ''
        self.workdir = ''
        self.version = version
        self.index = 0
        self.do_action = ''
        self.mode = ''

    async def start(self):
        if not await self.settings():
            await idle(exit=True)
            return

        async def do(value: str):
            self.index = 0
            self.do_action = value
            if value == '2':
                await self.read()
                # await self.cosCopy()
            elif value == '1':
                print('\n\033[31m要中断程序，请同时按下 ctrl+c\033[0m\n')
                await idle(self.read)

        await self.home(do)

    async def ctrl(self, response: Result):
        if response.code == 401:
            print(response.msg)
            await idle(exit=True)
            return
        if(not response.success or not response.data or not isinstance(response.data, dict)):
            print(response.msg or '请求任务时发生未知错误')
            if self.do_action == '2':
                return
            s = 10
            if isinstance(response.data, dict):
                s = response.data.get('seconds') or 10
            if s >= 5:
                await asyncio.sleep(s)
            return

        seconds = response.data.get('seconds')
        if seconds:
            if self.do_action == '2':
                return
            next = datetime.datetime.now()+datetime.timedelta(seconds=seconds)
            print(
                f'\r暂无任务，{seconds}秒后({next.strftime("%H:%M:%S")})将再次读取...', end=' ')
            await asyncio.sleep(seconds)
            return

        return True

    async def read(self):
        kwargs = dict(mode=self.mode)
        if self.mode == 'imitate':
            kwargs['user_agent'] = UserAgent().random
        # kwargs['mode']='webpublish'
        response = await self.query('/read', **kwargs)
        if not await self.ctrl(response):
            return

        self.index += 1
        mode = response.data.get('mode')
        video = response.data.get('video')
        session = response.data.get('session')
        account = response.data.get('account')

        _id = video.get('_id')
        status = video.get('status')
        title = video.get('title')
        record_caption = video.get('name')
        if account:
            record_caption += f'【\033[35m{account.get("name")}.{account.get("weight")}\033[0m】'

        print(f'\n{self.index}){record_caption}')

        # 首先下载视频
        files = [File(**x) for x in video.get('files')]
        result, cosRes = await self.downloadAndUpload(video.get('category_name'), files)

        kwargs = dict(_id=_id, title=title, status=status, mode=mode)

        if len(result) != len(files):
            await self.query('/updateStatus', **kwargs, childStatus='error', error="未能完全下载")
            return

        if mode == 'webupload' or status == 'export':
            res = await self.query('/updateStatus', **kwargs, childStatus='exported', cosRes=cosRes)
            if res.msg:
                print(f'\t{res.msg}')

        elif mode in ['webpublish', 'imitate', 'api']:
            if len(cosRes) != len(files):
                await self.query('/updateStatus', **kwargs, childStatus='error', error="未能完全上传")
                return

            kwargs.update({'childStatus': 'uploaded', 'cosRes': cosRes})

            if mode == 'webpublish':
                res = await self.query('/updateStatus', **kwargs)
                if res.msg:
                    print(f'\r\t{res.msg}')

            elif mode == 'imitate':
                kwargs['session'] = session
                print('\r\t发布中..', end=" ")
                res = await self.query('/updateStatus', **kwargs)
                if res.success:
                    print(f'\r\t\033[32m{res.msg}\033[0m')
                else:
                    print(f'\r\t\033[31m{res.msg}\033[0m')
            elif mode == 'api':
                print('\r\t发布中..', end=" ")
                if video.get('activity'):
                    print('\r\t模拟发布活动', end=" ")
                    res = await self.submit(
                        account.get('mobile'), 
                        account.get('password'), 
                        result[1],
                        video.get('title'), 
                        video.get('category'), 
                        video.get('source'), 
                        video.get('tag'),
                        uploadres_cos=cosRes.get('uploadres_cos'),
                        coverres_cos=cosRes.get('coverres_cos'),
                        coverres2_cos=cosRes.get('coverres2_cos'),
                        coverres1_cos=cosRes.get('coverres1_cos'),
                        activity=video.get('activity')
                    )
                else:
                    # res = await self.query('/updateStatus', **kwargs)
                    res = await self.api_publish(video, account.get('app_id'), account.get('app_token'), cosRes)
                kwargs['childStatus'] = 'completed'
                kwargs.update(res.data)
                await self.query('/updateStatus', **kwargs)
                code = kwargs.get('code')
                message = kwargs.get('message')
                if code == 0:
                    print(f'\r\t\033[32m{message}\033[0m')
                else:
                    print(f'\r\t\033[31m{message}\033[0m')

            else:
                print(f'\t无效的发布方式({mode})')

    async def downloadAndUpload(self, category_name: str, files: List[File]):
        savepath = os.path.join(
            self.workdir, datetime.datetime.now().strftime('%Y%m%d'))
        if not os.path.exists(savepath):
            os.mkdir(savepath)
        if category_name:
            savepath = os.path.join(savepath, category_name)
            if not os.path.exists(savepath):
                os.mkdir(savepath)

        cosRes = {}
        result = {}

        # 先下载files中的文件
        for file in files:
            filepath = os.path.join(savepath, file.name)
            allow_up = False
            if os.path.exists(filepath) and os.path.getsize(filepath) == file.size:
                print("\r\t%s(%s) 下载%d%%" %
                      (file.label, file.sizeStr, 100), end=" ")
                allow_up = True
                result[file.id] = filepath
            else:
                success = await self.download(filepath, file.src, lambda p, s, t: print("\r\t%s(%s) 下载%d%%" % (file.label, file.sizeStr, p), end=" "))
                if success and os.path.getsize(filepath) == file.size:
                    allow_up = True
                    result[file.id] = filepath
                else:
                    print('大小不一致' if success else '失败')
                    os.remove(filepath)
                    break

            if not allow_up:
                continue

            cos = file.cos
            if not cos:
                print('完成')
                continue

            accept = file.accept
            res = await self.to_cos(cos, filepath, accept, lambda p, s, t: print("\r\t%s(%s) 下载完成，上传%d%%" % (file.label, file.sizeStr, p), end=" "))
            if res.success:
                print('完成')
                if file.id == 0:
                    cosRes['uploadres_cos'] = res.data
                elif file.id == 1:
                    cosRes['coverres_cos'] = res.data  # coverres_cos 是横版封面
                elif file.id == 2:
                    cosRes['coverres2_cos'] = res.data  # coverres2_cos 是竖版封面
                elif file.id == 3:
                    cosRes['coverres1_cos'] = res.data  # 好像是分屏封面

            else:
                print(f'失败：{res.msg}')
                break
        return result, cosRes

    def __formatCosData(self, data: dict, is_video: bool):
        result = {'statusCode': 200}
        for key, value in data.items():
            if key == 'Location':
                Location: str = value.replace(
                    'http://', '').replace('https://', '')
                result['Location'] = Location
                if is_video:
                    arr = Location.split('.')
                    result['Bucket'] = arr[0]
                    result['Key'] = '/'.join(Location.split('/')[1:])
            elif key == 'ETag':
                result['ETag'] = f'"{value}"'
            elif key == 'x-cos-request-id':
                result['RequestId'] = value
            elif key == 'UploadId':
                result['UploadId'] = value

        """ if is_video:
            result['headers'] = {
                'content-type': 'application/xml', 'x-cos-request-id': result['RequestId']}
        else:
            result['headers'] = {
                'content-length': '0', 'etag': result['ETag'], 'x-cos-request-id': result['RequestId']} """
        return result

    async def api_publish(self, video: dict, app_id: str, app_token: str, cosRes: dict):
        title = video.get('title')
        uploadres_cos = cosRes.get('uploadres_cos')
        coverres_cos = cosRes.get('coverres_cos')
        coverres2_cos = cosRes.get('coverres2_cos')
        coverres1_cos = cosRes.get('coverres1_cos')
        data = {
            "app_id": app_id,
            "app_token": app_token,
            "title": title,
            "category": video.get('category'),
            "source": video.get('source'),
            "tag": video.get('tag'),
            "activity": video.get("activity") or "",
            "video_cos": self.__formatCosData(uploadres_cos, True),
            'coverinfo_cos': self.__formatCosData(coverres_cos, False)
        }
        if coverres2_cos:
            data['coverinfo2_cos'] = self.__formatCosData(coverres2_cos, False)

        if coverres1_cos:
            data['coverinfo1_cos'] = self.__formatCosData(coverres1_cos, False)

        headers = {'Content-Type': 'application/json'}
        url = 'https://mp.xiaozhuyouban.com/video/publish'

        response = requests.post(url, data=json.dumps(data), headers=headers)

        if(response.status_code == 200):
            value = response.json()
            code = value.get('code')
            message = value.get('message')
            return Result(code == 0, message, dict(code=code, message=message))
        else:
            message = f'发布请求异常({response.status_code})'
            return Result(False, message, dict(code=-1, message=message))
